@@ -16,7 +16,7 @@
 
 You'll typically use these to get started with the API:
 
-.. literalinclude:: storage_snippets.py
+.. literalinclude:: snippets.py
     :start-after: [START storage_get_started]
     :end-before: [END storage_get_started]
 
@@ -38,3 +38,6 @@ from google.cloud.storage.batch import Batch
 from google.cloud.storage.blob import Blob
 from google.cloud.storage.bucket import Bucket
 from google.cloud.storage.client import Client
+
+
+__all__ = ['__version__', 'Batch', 'Blob', 'Bucket', 'Client']
