@@ -34,7 +34,11 @@ import os
 import time
 import warnings
 
+from six.moves.urllib.parse import parse_qsl
 from six.moves.urllib.parse import quote
+from six.moves.urllib.parse import urlencode
+from six.moves.urllib.parse import urlsplit
+from six.moves.urllib.parse import urlunsplit
 
 from google import resumable_media
 from google.resumable_media.requests import ChunkedDownload
@@ -220,6 +224,16 @@ class Blob(_PropertyMixin):
         """The client bound to this blob."""
         return self.bucket.client
 
+    @property
+    def user_project(self):
+        """Project ID billed for API requests made via this blob.
+
+        Derived from bucket's value.
+
+        :rtype: str
+        """
+        return self.bucket.user_project
+
     @property
     def public_url(self):
         """The public URL for this blob's object.
@@ -319,6 +333,9 @@ class Blob(_PropertyMixin):
     def exists(self, client=None):
         """Determines whether or not this blob exists.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -328,10 +345,14 @@ class Blob(_PropertyMixin):
         :returns: True if the blob exists in Cloud Storage.
         """
         client = self._require_client(client)
+        # We only need the status code (200 or not) so we seek to
+        # minimize the returned payload.
+        query_params = {'fields': 'name'}
+
+        if self.user_project is not None:
+            query_params['userProject'] = self.user_project
+
         try:
-            # We only need the status code (200 or not) so we seek to
-            # minimize the returned payload.
-            query_params = {'fields': 'name'}
             # We intentionally pass `_target_object=None` since fields=name
             # would limit the local properties.
             client._connection.api_request(
@@ -347,6 +368,9 @@ class Blob(_PropertyMixin):
     def delete(self, client=None):
         """Deletes a blob from Cloud Storage.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -385,13 +409,19 @@ class Blob(_PropertyMixin):
         :rtype: str
         :returns: The download URL for the current blob.
         """
+        name_value_pairs = []
         if self.media_link is None:
-            download_url = _DOWNLOAD_URL_TEMPLATE.format(path=self.path)
+            base_url = _DOWNLOAD_URL_TEMPLATE.format(path=self.path)
             if self.generation is not None:
-                download_url += u'&generation={:d}'.format(self.generation)
-            return download_url
+                name_value_pairs.append(
+                    ('generation', '{:d}'.format(self.generation)))
         else:
-            return self.media_link
+            base_url = self.media_link
+
+        if self.user_project is not None:
+            name_value_pairs.append(('userProject', self.user_project))
+
+        return _add_query_parameters(base_url, name_value_pairs)
 
     def _do_download(self, transport, file_obj, download_url, headers):
         """Perform a download without any error handling.
@@ -446,6 +476,9 @@ class Blob(_PropertyMixin):
         `google-resumable-media`_. For example, this library allows
         downloading **parts** of a blob rather than the whole thing.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type file_obj: file
         :param file_obj: A file handle to which to write the blob's data.
 
@@ -458,8 +491,9 @@ class Blob(_PropertyMixin):
         """
         download_url = self._get_download_url()
         headers = _get_encryption_headers(self._encryption_key)
-        transport = self._get_transport(client)
+        headers['accept-encoding'] = 'gzip'
 
+        transport = self._get_transport(client)
         try:
             self._do_download(transport, file_obj, download_url, headers)
         except resumable_media.InvalidResponse as exc:
@@ -468,6 +502,9 @@ class Blob(_PropertyMixin):
     def download_to_filename(self, filename, client=None):
         """Download the contents of this blob into a named file.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type filename: str
         :param filename: A filename to be passed to ``open``.
 
@@ -478,8 +515,13 @@ class Blob(_PropertyMixin):
 
         :raises: :class:`google.cloud.exceptions.NotFound`
         """
-        with open(filename, 'wb') as file_obj:
-            self.download_to_file(file_obj, client=client)
+        try:
+            with open(filename, 'wb') as file_obj:
+                self.download_to_file(file_obj, client=client)
+        except resumable_media.DataCorruption as exc:
+            # Delete the corrupt downloaded file.
+            os.remove(filename)
+            raise
 
         updated = self.updated
         if updated is not None:
@@ -489,6 +531,9 @@ class Blob(_PropertyMixin):
     def download_as_string(self, client=None):
         """Download the contents of this blob as a string.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -637,8 +682,14 @@ class Blob(_PropertyMixin):
         info = self._get_upload_arguments(content_type)
         headers, object_metadata, content_type = info
 
-        upload_url = _MULTIPART_URL_TEMPLATE.format(
+        base_url = _MULTIPART_URL_TEMPLATE.format(
             bucket_path=self.bucket.path)
+        name_value_pairs = []
+
+        if self.user_project is not None:
+            name_value_pairs.append(('userProject', self.user_project))
+
+        upload_url = _add_query_parameters(base_url, name_value_pairs)
         upload = MultipartUpload(upload_url, headers=headers)
 
         if num_retries is not None:
@@ -709,8 +760,14 @@ class Blob(_PropertyMixin):
         if extra_headers is not None:
             headers.update(extra_headers)
 
-        upload_url = _RESUMABLE_URL_TEMPLATE.format(
+        base_url = _RESUMABLE_URL_TEMPLATE.format(
             bucket_path=self.bucket.path)
+        name_value_pairs = []
+
+        if self.user_project is not None:
+            name_value_pairs.append(('userProject', self.user_project))
+
+        upload_url = _add_query_parameters(base_url, name_value_pairs)
         upload = ResumableUpload(upload_url, chunk_size, headers=headers)
 
         if num_retries is not None:
@@ -847,6 +904,9 @@ class Blob(_PropertyMixin):
         For more fine-grained over the upload process, check out
         `google-resumable-media`_.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type file_obj: file
         :param file_obj: A file handle open for reading.
 
@@ -910,6 +970,9 @@ class Blob(_PropertyMixin):
            `lifecycle <https://cloud.google.com/storage/docs/lifecycle>`_
            API documents for details.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type filename: str
         :param filename: The path to the file.
 
@@ -942,6 +1005,9 @@ class Blob(_PropertyMixin):
            `lifecycle <https://cloud.google.com/storage/docs/lifecycle>`_
            API documents for details.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type data: bytes or str
         :param data: The data to store in this blob.  If the value is
                      text, it will be encoded as UTF-8.
@@ -1003,6 +1069,9 @@ class Blob(_PropertyMixin):
         If :attr:`encryption_key` is set, the blob will be encrypted with
         a `customer-supplied`_ encryption key.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type size: int
         :param size: (Optional). The maximum number of bytes that can be
                      uploaded using this session. If the size is not known
@@ -1054,6 +1123,9 @@ class Blob(_PropertyMixin):
         See
         https://cloud.google.com/storage/docs/json_api/v1/objects/getIamPolicy
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -1064,9 +1136,16 @@ class Blob(_PropertyMixin):
                   the ``getIamPolicy`` API request.
         """
         client = self._require_client(client)
+
+        query_params = {}
+
+        if self.user_project is not None:
+            query_params['userProject'] = self.user_project
+
         info = client._connection.api_request(
             method='GET',
             path='%s/iam' % (self.path,),
+            query_params=query_params,
             _target_object=None)
         return Policy.from_api_repr(info)
 
@@ -1076,6 +1155,9 @@ class Blob(_PropertyMixin):
         See
         https://cloud.google.com/storage/docs/json_api/v1/objects/setIamPolicy
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type policy: :class:`google.cloud.iam.Policy`
         :param policy: policy instance used to update bucket's IAM policy.
 
@@ -1089,11 +1171,18 @@ class Blob(_PropertyMixin):
                   the ``setIamPolicy`` API request.
         """
         client = self._require_client(client)
+
+        query_params = {}
+
+        if self.user_project is not None:
+            query_params['userProject'] = self.user_project
+
         resource = policy.to_api_repr()
         resource['resourceId'] = self.path
         info = client._connection.api_request(
             method='PUT',
             path='%s/iam' % (self.path,),
+            query_params=query_params,
             data=resource,
             _target_object=None)
         return Policy.from_api_repr(info)
@@ -1104,6 +1193,9 @@ class Blob(_PropertyMixin):
         See
         https://cloud.google.com/storage/docs/json_api/v1/objects/testIamPermissions
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type permissions: list of string
         :param permissions: the permissions to check
 
@@ -1117,12 +1209,17 @@ class Blob(_PropertyMixin):
                   request.
         """
         client = self._require_client(client)
-        query = {'permissions': permissions}
+        query_params = {'permissions': permissions}
+
+        if self.user_project is not None:
+            query_params['userProject'] = self.user_project
+
         path = '%s/iam/testPermissions' % (self.path,)
         resp = client._connection.api_request(
             method='GET',
             path=path,
-            query_params=query)
+            query_params=query_params)
+
         return resp.get('permissions', [])
 
     def make_public(self, client=None):
@@ -1139,6 +1236,9 @@ class Blob(_PropertyMixin):
     def compose(self, sources, client=None):
         """Concatenate source blobs into this one.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type sources: list of :class:`Blob`
         :param sources: blobs whose contents will be composed into this blob.
 
@@ -1152,19 +1252,31 @@ class Blob(_PropertyMixin):
         """
         if self.content_type is None:
             raise ValueError("Destination 'content_type' not set.")
+
         client = self._require_client(client)
+        query_params = {}
+
+        if self.user_project is not None:
+            query_params['userProject'] = self.user_project
+
         request = {
             'sourceObjects': [{'name': source.name} for source in sources],
             'destination': self._properties.copy(),
         }
         api_response = client._connection.api_request(
-            method='POST', path=self.path + '/compose', data=request,
+            method='POST',
+            path=self.path + '/compose',
+            query_params=query_params,
+            data=request,
             _target_object=self)
         self._set_properties(api_response)
 
     def rewrite(self, source, token=None, client=None):
         """Rewrite source blob into this one.
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type source: :class:`Blob`
         :param source: blob whose contents will be rewritten into this blob.
 
@@ -1190,14 +1302,20 @@ class Blob(_PropertyMixin):
         headers.update(_get_encryption_headers(
             source._encryption_key, source=True))
 
+        query_params = {}
+
         if token:
-            query_params = {'rewriteToken': token}
-        else:
-            query_params = {}
+            query_params['rewriteToken'] = token
+
+        if self.user_project is not None:
+            query_params['userProject'] = self.user_project
 
         api_response = client._connection.api_request(
-            method='POST', path=source.path + '/rewriteTo' + self.path,
-            query_params=query_params, data=self._properties, headers=headers,
+            method='POST',
+            path=source.path + '/rewriteTo' + self.path,
+            query_params=query_params,
+            data=self._properties,
+            headers=headers,
             _target_object=self)
         rewritten = int(api_response['totalBytesRewritten'])
         size = int(api_response['objectSize'])
@@ -1217,6 +1335,9 @@ class Blob(_PropertyMixin):
         See
         https://cloud.google.com/storage/docs/per-object-storage-class
 
+        If :attr:`user_project` is set on the bucket, bills the API request
+        to that project.
+
         :type new_class: str
         :param new_class: new storage class for the object
 
@@ -1228,13 +1349,22 @@ class Blob(_PropertyMixin):
             raise ValueError("Invalid storage class: %s" % (new_class,))
 
         client = self._require_client(client)
+
+        query_params = {}
+
+        if self.user_project is not None:
+            query_params['userProject'] = self.user_project
+
         headers = _get_encryption_headers(self._encryption_key)
         headers.update(_get_encryption_headers(
             self._encryption_key, source=True))
 
         api_response = client._connection.api_request(
-            method='POST', path=self.path + '/rewriteTo' + self.path,
-            data={'storageClass': new_class}, headers=headers,
+            method='POST',
+            path=self.path + '/rewriteTo' + self.path,
+            query_params=query_params,
+            data={'storageClass': new_class},
+            headers=headers,
             _target_object=self)
         self._set_properties(api_response['resource'])
 
@@ -1603,3 +1733,24 @@ def _raise_from_invalid_response(error):
              to the failed status code
     """
     raise exceptions.from_http_response(error.response)
+
+
+def _add_query_parameters(base_url, name_value_pairs):
+    """Add one query parameter to a base URL.
+
+    :type base_url: string
+    :param base_url: Base URL (may already contain query parameters)
+
+    :type name_value_pairs: list of (string, string) tuples.
+    :param name_value_pairs: Names and values of the query parameters to add
+
+    :rtype: string
+    :returns: URL with additional query strings appended.
+    """
+    if len(name_value_pairs) == 0:
+        return base_url
+
+    scheme, netloc, path, query, frag = urlsplit(base_url)
+    query = parse_qsl(query)
+    query.extend(name_value_pairs)
+    return urlunsplit((scheme, netloc, path, urlencode(query), frag))
