@@ -33,13 +33,21 @@ def _create_signing_credentials():
 
 class Test_Bucket(unittest.TestCase):
 
-    def _make_one(self, client=None, name=None, properties=None):
+    @staticmethod
+    def _get_target_class():
         from google.cloud.storage.bucket import Bucket
+        return Bucket
 
+    def _make_one(
+            self, client=None, name=None, properties=None, user_project=None):
         if client is None:
             connection = _Connection()
             client = _Client(connection)
-        bucket = Bucket(client, name=name)
+        if user_project is None:
+            bucket = self._get_target_class()(client, name=name)
+        else:
+            bucket = self._get_target_class()(
+                client, name=name, user_project=user_project)
         bucket._properties = properties or {}
         return bucket
 
@@ -53,6 +61,21 @@ class Test_Bucket(unittest.TestCase):
         self.assertIs(bucket._acl.bucket, bucket)
         self.assertFalse(bucket._default_object_acl.loaded)
         self.assertIs(bucket._default_object_acl.bucket, bucket)
+        self.assertIsNone(bucket.user_project)
+
+    def test_ctor_w_user_project(self):
+        NAME = 'name'
+        USER_PROJECT = 'user-project-123'
+        connection = _Connection()
+        client = _Client(connection)
+        bucket = self._make_one(client, name=NAME, user_project=USER_PROJECT)
+        self.assertEqual(bucket.name, NAME)
+        self.assertEqual(bucket._properties, {})
+        self.assertEqual(bucket.user_project, USER_PROJECT)
+        self.assertFalse(bucket._acl.loaded)
+        self.assertIs(bucket._acl.bucket, bucket)
+        self.assertFalse(bucket._default_object_acl.loaded)
+        self.assertIs(bucket._default_object_acl.bucket, bucket)
 
     def test_blob(self):
         from google.cloud.storage.blob import Blob
@@ -72,10 +95,67 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(blob.chunk_size, CHUNK_SIZE)
         self.assertEqual(blob._encryption_key, KEY)
 
+    def test_notification_defaults(self):
+        from google.cloud.storage.notification import BucketNotification
+        from google.cloud.storage.notification import NONE_PAYLOAD_FORMAT
+
+        PROJECT = 'PROJECT'
+        BUCKET_NAME = 'BUCKET_NAME'
+        TOPIC_NAME = 'TOPIC_NAME'
+        client = _Client(_Connection(), project=PROJECT)
+        bucket = self._make_one(client, name=BUCKET_NAME)
+
+        notification = bucket.notification(TOPIC_NAME)
+
+        self.assertIsInstance(notification, BucketNotification)
+        self.assertIs(notification.bucket, bucket)
+        self.assertEqual(notification.topic_project, PROJECT)
+        self.assertIsNone(notification.custom_attributes)
+        self.assertIsNone(notification.event_types)
+        self.assertIsNone(notification.blob_name_prefix)
+        self.assertEqual(notification.payload_format, NONE_PAYLOAD_FORMAT)
+
+    def test_notification_explicit(self):
+        from google.cloud.storage.notification import (
+            BucketNotification,
+            OBJECT_FINALIZE_EVENT_TYPE,
+            OBJECT_DELETE_EVENT_TYPE,
+            JSON_API_V1_PAYLOAD_FORMAT)
+
+        PROJECT = 'PROJECT'
+        BUCKET_NAME = 'BUCKET_NAME'
+        TOPIC_NAME = 'TOPIC_NAME'
+        TOPIC_ALT_PROJECT = 'topic-project-456'
+        CUSTOM_ATTRIBUTES = {
+            'attr1': 'value1',
+            'attr2': 'value2',
+        }
+        EVENT_TYPES = [OBJECT_FINALIZE_EVENT_TYPE, OBJECT_DELETE_EVENT_TYPE]
+        BLOB_NAME_PREFIX = 'blob-name-prefix/'
+        client = _Client(_Connection(), project=PROJECT)
+        bucket = self._make_one(client, name=BUCKET_NAME)
+
+        notification = bucket.notification(
+            TOPIC_NAME,
+            topic_project=TOPIC_ALT_PROJECT,
+            custom_attributes=CUSTOM_ATTRIBUTES,
+            event_types=EVENT_TYPES,
+            blob_name_prefix=BLOB_NAME_PREFIX,
+            payload_format=JSON_API_V1_PAYLOAD_FORMAT,
+        )
+
+        self.assertIsInstance(notification, BucketNotification)
+        self.assertIs(notification.bucket, bucket)
+        self.assertEqual(notification.topic_project, TOPIC_ALT_PROJECT)
+        self.assertEqual(notification.custom_attributes, CUSTOM_ATTRIBUTES)
+        self.assertEqual(notification.event_types, EVENT_TYPES)
+        self.assertEqual(notification.blob_name_prefix, BLOB_NAME_PREFIX)
+        self.assertEqual(
+            notification.payload_format, JSON_API_V1_PAYLOAD_FORMAT)
+
     def test_bucket_name_value(self):
-        bucket_name = 'testing123'
-        mixin = self._make_one(name=bucket_name)
-        self.assertEqual(mixin.name, bucket_name)
+        BUCKET_NAME = 'bucket-name'
+        bucket = self._make_one(name=BUCKET_NAME)
 
         bad_start_bucket_name = '/testing123'
         with self.assertRaises(ValueError):
@@ -85,6 +165,13 @@ class Test_Bucket(unittest.TestCase):
         with self.assertRaises(ValueError):
             self._make_one(name=bad_end_bucket_name)
 
+    def test_user_project(self):
+        BUCKET_NAME = 'name'
+        USER_PROJECT = 'user-project-123'
+        bucket = self._make_one(name=BUCKET_NAME)
+        bucket._user_project = USER_PROJECT
+        self.assertEqual(bucket.user_project, USER_PROJECT)
+
     def test_exists_miss(self):
         from google.cloud.exceptions import NotFound
 
@@ -112,7 +199,9 @@ class Test_Bucket(unittest.TestCase):
         expected_cw = [((), expected_called_kwargs)]
         self.assertEqual(_FakeConnection._called_with, expected_cw)
 
-    def test_exists_hit(self):
+    def test_exists_hit_w_user_project(self):
+        USER_PROJECT = 'user-project-123'
+
         class _FakeConnection(object):
 
             _called_with = []
@@ -124,7 +213,7 @@ class Test_Bucket(unittest.TestCase):
                 return object()
 
         BUCKET_NAME = 'bucket-name'
-        bucket = self._make_one(name=BUCKET_NAME)
+        bucket = self._make_one(name=BUCKET_NAME, user_project=USER_PROJECT)
         client = _Client(_FakeConnection)
         self.assertTrue(bucket.exists(client=client))
         expected_called_kwargs = {
@@ -132,17 +221,29 @@ class Test_Bucket(unittest.TestCase):
             'path': bucket.path,
             'query_params': {
                 'fields': 'name',
+                'userProject': USER_PROJECT,
             },
             '_target_object': None,
         }
         expected_cw = [((), expected_called_kwargs)]
         self.assertEqual(_FakeConnection._called_with, expected_cw)
 
+    def test_create_w_user_project(self):
+        PROJECT = 'PROJECT'
+        BUCKET_NAME = 'bucket-name'
+        USER_PROJECT = 'user-project-123'
+        connection = _Connection()
+        client = _Client(connection, project=PROJECT)
+        bucket = self._make_one(client, BUCKET_NAME, user_project=USER_PROJECT)
+
+        with self.assertRaises(ValueError):
+            bucket.create()
+
     def test_create_hit(self):
+        PROJECT = 'PROJECT'
         BUCKET_NAME = 'bucket-name'
         DATA = {'name': BUCKET_NAME}
         connection = _Connection(DATA)
-        PROJECT = 'PROJECT'
         client = _Client(connection, project=PROJECT)
         bucket = self._make_one(client=client, name=BUCKET_NAME)
         bucket.create()
@@ -176,6 +277,7 @@ class Test_Bucket(unittest.TestCase):
             'location': LOCATION,
             'storageClass': STORAGE_CLASS,
             'versioning': {'enabled': True},
+            'billing': {'requesterPays': True},
             'labels': LABELS,
         }
         connection = _Connection(DATA)
@@ -186,6 +288,7 @@ class Test_Bucket(unittest.TestCase):
         bucket.location = LOCATION
         bucket.storage_class = STORAGE_CLASS
         bucket.versioning_enabled = True
+        bucket.requester_pays = True
         bucket.labels = LABELS
         bucket.create()
 
@@ -232,18 +335,20 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(kw['method'], 'GET')
         self.assertEqual(kw['path'], '/b/%s/o/%s' % (NAME, NONESUCH))
 
-    def test_get_blob_hit(self):
+    def test_get_blob_hit_w_user_project(self):
         NAME = 'name'
         BLOB_NAME = 'blob-name'
+        USER_PROJECT = 'user-project-123'
         connection = _Connection({'name': BLOB_NAME})
         client = _Client(connection)
-        bucket = self._make_one(name=NAME)
+        bucket = self._make_one(name=NAME, user_project=USER_PROJECT)
         blob = bucket.get_blob(BLOB_NAME, client=client)
         self.assertIs(blob.bucket, bucket)
         self.assertEqual(blob.name, BLOB_NAME)
         kw, = connection._requested
         self.assertEqual(kw['method'], 'GET')
         self.assertEqual(kw['path'], '/b/%s/o/%s' % (NAME, BLOB_NAME))
+        self.assertEqual(kw['query_params'], {'userProject': USER_PROJECT})
 
     def test_get_blob_hit_with_kwargs(self):
         from google.cloud.storage.blob import _get_encryption_headers
@@ -281,8 +386,9 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(kw['path'], '/b/%s/o' % NAME)
         self.assertEqual(kw['query_params'], {'projection': 'noAcl'})
 
-    def test_list_blobs_w_all_arguments(self):
+    def test_list_blobs_w_all_arguments_and_user_project(self):
         NAME = 'name'
+        USER_PROJECT = 'user-project-123'
         MAX_RESULTS = 10
         PAGE_TOKEN = 'ABCD'
         PREFIX = 'subfolder'
@@ -298,10 +404,11 @@ class Test_Bucket(unittest.TestCase):
             'versions': VERSIONS,
             'projection': PROJECTION,
             'fields': FIELDS,
+            'userProject': USER_PROJECT,
         }
         connection = _Connection({'items': []})
         client = _Client(connection)
-        bucket = self._make_one(name=NAME)
+        bucket = self._make_one(name=NAME, user_project=USER_PROJECT)
         iterator = bucket.list_blobs(
             max_results=MAX_RESULTS,
             page_token=PAGE_TOKEN,
@@ -332,6 +439,58 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(kw['path'], '/b/%s/o' % NAME)
         self.assertEqual(kw['query_params'], {'projection': 'noAcl'})
 
+    def test_list_notifications(self):
+        from google.cloud.storage.notification import BucketNotification
+        from google.cloud.storage.notification import _TOPIC_REF_FMT
+        from google.cloud.storage.notification import (
+            JSON_API_V1_PAYLOAD_FORMAT, NONE_PAYLOAD_FORMAT)
+
+        NAME = 'name'
+
+        topic_refs = [
+            ('my-project-123', 'topic-1'),
+            ('other-project-456', 'topic-2'),
+        ]
+
+        resources = [{
+            'topic': _TOPIC_REF_FMT.format(*topic_refs[0]),
+            'id': '1',
+            'etag': 'DEADBEEF',
+            'selfLink': 'https://example.com/notification/1',
+            'payload_format': NONE_PAYLOAD_FORMAT,
+        }, {
+            'topic': _TOPIC_REF_FMT.format(*topic_refs[1]),
+            'id': '2',
+            'etag': 'FACECABB',
+            'selfLink': 'https://example.com/notification/2',
+            'payload_format': JSON_API_V1_PAYLOAD_FORMAT,
+        }]
+        connection = _Connection({'items': resources})
+        client = _Client(connection)
+        bucket = self._make_one(client=client, name=NAME)
+
+        notifications = list(bucket.list_notifications())
+
+        self.assertEqual(len(notifications), len(resources))
+        for notification, resource, topic_ref in zip(
+            notifications, resources, topic_refs):
+            self.assertIsInstance(notification, BucketNotification)
+            self.assertEqual(notification.topic_project, topic_ref[0])
+            self.assertEqual(notification.topic_name, topic_ref[1])
+            self.assertEqual(notification.notification_id, resource['id'])
+            self.assertEqual(notification.etag, resource['etag'])
+            self.assertEqual(notification.self_link, resource['selfLink'])
+            self.assertEqual(
+                notification.custom_attributes,
+                resource.get('custom_attributes'))
+            self.assertEqual(
+                notification.event_types, resource.get('event_types'))
+            self.assertEqual(
+                notification.blob_name_prefix,
+                resource.get('blob_name_prefix'))
+            self.assertEqual(
+                notification.payload_format, resource.get('payload_format'))
+
     def test_delete_miss(self):
         from google.cloud.exceptions import NotFound
 
@@ -343,23 +502,27 @@ class Test_Bucket(unittest.TestCase):
         expected_cw = [{
             'method': 'DELETE',
             'path': bucket.path,
+            'query_params': {},
             '_target_object': None,
         }]
         self.assertEqual(connection._deleted_buckets, expected_cw)
 
-    def test_delete_hit(self):
+    def test_delete_hit_with_user_project(self):
         NAME = 'name'
+        USER_PROJECT = 'user-project-123'
         GET_BLOBS_RESP = {'items': []}
         connection = _Connection(GET_BLOBS_RESP)
         connection._delete_bucket = True
         client = _Client(connection)
-        bucket = self._make_one(client=client, name=NAME)
+        bucket = self._make_one(
+            client=client, name=NAME, user_project=USER_PROJECT)
         result = bucket.delete(force=True)
         self.assertIsNone(result)
         expected_cw = [{
             'method': 'DELETE',
             'path': bucket.path,
             '_target_object': None,
+            'query_params': {'userProject': USER_PROJECT},
         }]
         self.assertEqual(connection._deleted_buckets, expected_cw)
 
@@ -384,6 +547,7 @@ class Test_Bucket(unittest.TestCase):
         expected_cw = [{
             'method': 'DELETE',
             'path': bucket.path,
+            'query_params': {},
             '_target_object': None,
         }]
         self.assertEqual(connection._deleted_buckets, expected_cw)
@@ -402,6 +566,7 @@ class Test_Bucket(unittest.TestCase):
         expected_cw = [{
             'method': 'DELETE',
             'path': bucket.path,
+            'query_params': {},
             '_target_object': None,
         }]
         self.assertEqual(connection._deleted_buckets, expected_cw)
@@ -438,18 +603,22 @@ class Test_Bucket(unittest.TestCase):
         kw, = connection._requested
         self.assertEqual(kw['method'], 'DELETE')
         self.assertEqual(kw['path'], '/b/%s/o/%s' % (NAME, NONESUCH))
+        self.assertEqual(kw['query_params'], {})
 
-    def test_delete_blob_hit(self):
+    def test_delete_blob_hit_with_user_project(self):
         NAME = 'name'
         BLOB_NAME = 'blob-name'
+        USER_PROJECT = 'user-project-123'
         connection = _Connection({})
         client = _Client(connection)
-        bucket = self._make_one(client=client, name=NAME)
+        bucket = self._make_one(
+            client=client, name=NAME, user_project=USER_PROJECT)
         result = bucket.delete_blob(BLOB_NAME)
         self.assertIsNone(result)
         kw, = connection._requested
         self.assertEqual(kw['method'], 'DELETE')
         self.assertEqual(kw['path'], '/b/%s/o/%s' % (NAME, BLOB_NAME))
+        self.assertEqual(kw['query_params'], {'userProject': USER_PROJECT})
 
     def test_delete_blobs_empty(self):
         NAME = 'name'
@@ -459,17 +628,20 @@ class Test_Bucket(unittest.TestCase):
         bucket.delete_blobs([])
         self.assertEqual(connection._requested, [])
 
-    def test_delete_blobs_hit(self):
+    def test_delete_blobs_hit_w_user_project(self):
         NAME = 'name'
         BLOB_NAME = 'blob-name'
+        USER_PROJECT = 'user-project-123'
         connection = _Connection({})
         client = _Client(connection)
-        bucket = self._make_one(client=client, name=NAME)
+        bucket = self._make_one(
+            client=client, name=NAME, user_project=USER_PROJECT)
         bucket.delete_blobs([BLOB_NAME])
         kw = connection._requested
         self.assertEqual(len(kw), 1)
         self.assertEqual(kw[0]['method'], 'DELETE')
         self.assertEqual(kw[0]['path'], '/b/%s/o/%s' % (NAME, BLOB_NAME))
+        self.assertEqual(kw[0]['query_params'], {'userProject': USER_PROJECT})
 
     def test_delete_blobs_miss_no_on_error(self):
         from google.cloud.exceptions import NotFound
@@ -527,6 +699,7 @@ class Test_Bucket(unittest.TestCase):
                                                      DEST, BLOB_NAME)
         self.assertEqual(kw['method'], 'POST')
         self.assertEqual(kw['path'], COPY_PATH)
+        self.assertEqual(kw['query_params'], {})
 
     def test_copy_blobs_preserve_acl(self):
         from google.cloud.storage.acl import ObjectACL
@@ -558,14 +731,17 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(len(kw), 2)
         self.assertEqual(kw[0]['method'], 'POST')
         self.assertEqual(kw[0]['path'], COPY_PATH)
+        self.assertEqual(kw[0]['query_params'], {})
         self.assertEqual(kw[1]['method'], 'PATCH')
         self.assertEqual(kw[1]['path'], NEW_BLOB_PATH)
+        self.assertEqual(kw[1]['query_params'], {'projection': 'full'})
 
-    def test_copy_blobs_w_name(self):
+    def test_copy_blobs_w_name_and_user_project(self):
         SOURCE = 'source'
         DEST = 'dest'
         BLOB_NAME = 'blob-name'
         NEW_NAME = 'new_name'
+        USER_PROJECT = 'user-project-123'
 
         class _Blob(object):
             name = BLOB_NAME
@@ -573,7 +749,8 @@ class Test_Bucket(unittest.TestCase):
 
         connection = _Connection({})
         client = _Client(connection)
-        source = self._make_one(client=client, name=SOURCE)
+        source = self._make_one(
+            client=client, name=SOURCE, user_project=USER_PROJECT)
         dest = self._make_one(client=client, name=DEST)
         blob = _Blob()
         new_blob = source.copy_blob(blob, dest, NEW_NAME)
@@ -584,6 +761,7 @@ class Test_Bucket(unittest.TestCase):
                                                      DEST, NEW_NAME)
         self.assertEqual(kw['method'], 'POST')
         self.assertEqual(kw['path'], COPY_PATH)
+        self.assertEqual(kw['query_params'], {'userProject': USER_PROJECT})
 
     def test_rename_blob(self):
         BUCKET_NAME = 'BUCKET_NAME'
@@ -916,6 +1094,24 @@ class Test_Bucket(unittest.TestCase):
         bucket.versioning_enabled = True
         self.assertTrue(bucket.versioning_enabled)
 
+    def test_requester_pays_getter_missing(self):
+        NAME = 'name'
+        bucket = self._make_one(name=NAME)
+        self.assertEqual(bucket.requester_pays, False)
+
+    def test_requester_pays_getter(self):
+        NAME = 'name'
+        before = {'billing': {'requesterPays': True}}
+        bucket = self._make_one(name=NAME, properties=before)
+        self.assertEqual(bucket.requester_pays, True)
+
+    def test_requester_pays_setter(self):
+        NAME = 'name'
+        bucket = self._make_one(name=NAME)
+        self.assertFalse(bucket.requester_pays)
+        bucket.requester_pays = True
+        self.assertTrue(bucket.requester_pays)
+
     def test_configure_website_defaults(self):
         NAME = 'name'
         UNSET = {'website': {'mainPageSuffix': None,
@@ -984,6 +1180,40 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(len(kw), 1)
         self.assertEqual(kw[0]['method'], 'GET')
         self.assertEqual(kw[0]['path'], '%s/iam' % (PATH,))
+        self.assertEqual(kw[0]['query_params'], {})
+
+    def test_get_iam_policy_w_user_project(self):
+        from google.cloud.iam import Policy
+
+        NAME = 'name'
+        USER_PROJECT = 'user-project-123'
+        PATH = '/b/%s' % (NAME,)
+        ETAG = 'DEADBEEF'
+        VERSION = 17
+        RETURNED = {
+            'resourceId': PATH,
+            'etag': ETAG,
+            'version': VERSION,
+            'bindings': [],
+        }
+        EXPECTED = {}
+        connection = _Connection(RETURNED)
+        client = _Client(connection, None)
+        bucket = self._make_one(
+            client=client, name=NAME, user_project=USER_PROJECT)
+
+        policy = bucket.get_iam_policy()
+
+        self.assertIsInstance(policy, Policy)
+        self.assertEqual(policy.etag, RETURNED['etag'])
+        self.assertEqual(policy.version, RETURNED['version'])
+        self.assertEqual(dict(policy), EXPECTED)
+
+        kw = connection._requested
+        self.assertEqual(len(kw), 1)
+        self.assertEqual(kw[0]['method'], 'GET')
+        self.assertEqual(kw[0]['path'], '%s/iam' % (PATH,))
+        self.assertEqual(kw[0]['query_params'], {'userProject': USER_PROJECT})
 
     def test_set_iam_policy(self):
         import operator
@@ -1030,6 +1260,66 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(len(kw), 1)
         self.assertEqual(kw[0]['method'], 'PUT')
         self.assertEqual(kw[0]['path'], '%s/iam' % (PATH,))
+        self.assertEqual(kw[0]['query_params'], {})
+        sent = kw[0]['data']
+        self.assertEqual(sent['resourceId'], PATH)
+        self.assertEqual(len(sent['bindings']), len(BINDINGS))
+        key = operator.itemgetter('role')
+        for found, expected in zip(
+            sorted(sent['bindings'], key=key),
+            sorted(BINDINGS, key=key)):
+            self.assertEqual(found['role'], expected['role'])
+            self.assertEqual(
+                sorted(found['members']), sorted(expected['members']))
+
+    def test_set_iam_policy_w_user_project(self):
+        import operator
+        from google.cloud.storage.iam import STORAGE_OWNER_ROLE
+        from google.cloud.storage.iam import STORAGE_EDITOR_ROLE
+        from google.cloud.storage.iam import STORAGE_VIEWER_ROLE
+        from google.cloud.iam import Policy
+
+        NAME = 'name'
+        USER_PROJECT = 'user-project-123'
+        PATH = '/b/%s' % (NAME,)
+        ETAG = 'DEADBEEF'
+        VERSION = 17
+        OWNER1 = 'user:phred@example.com'
+        OWNER2 = 'group:cloud-logs@google.com'
+        EDITOR1 = 'domain:google.com'
+        EDITOR2 = 'user:phred@example.com'
+        VIEWER1 = 'serviceAccount:1234-abcdef@service.example.com'
+        VIEWER2 = 'user:phred@example.com'
+        BINDINGS = [
+            {'role': STORAGE_OWNER_ROLE, 'members': [OWNER1, OWNER2]},
+            {'role': STORAGE_EDITOR_ROLE, 'members': [EDITOR1, EDITOR2]},
+            {'role': STORAGE_VIEWER_ROLE, 'members': [VIEWER1, VIEWER2]},
+        ]
+        RETURNED = {
+            'etag': ETAG,
+            'version': VERSION,
+            'bindings': BINDINGS,
+        }
+        policy = Policy()
+        for binding in BINDINGS:
+            policy[binding['role']] = binding['members']
+
+        connection = _Connection(RETURNED)
+        client = _Client(connection, None)
+        bucket = self._make_one(
+            client=client, name=NAME, user_project=USER_PROJECT)
+
+        returned = bucket.set_iam_policy(policy)
+
+        self.assertEqual(returned.etag, ETAG)
+        self.assertEqual(returned.version, VERSION)
+        self.assertEqual(dict(returned), dict(policy))
+
+        kw = connection._requested
+        self.assertEqual(len(kw), 1)
+        self.assertEqual(kw[0]['method'], 'PUT')
+        self.assertEqual(kw[0]['path'], '%s/iam' % (PATH,))
+        self.assertEqual(kw[0]['query_params'], {'userProject': USER_PROJECT})
         sent = kw[0]['data']
         self.assertEqual(sent['resourceId'], PATH)
         self.assertEqual(len(sent['bindings']), len(BINDINGS))
@@ -1069,6 +1359,38 @@ class Test_Bucket(unittest.TestCase):
         self.assertEqual(kw[0]['path'], '%s/iam/testPermissions' % (PATH,))
         self.assertEqual(kw[0]['query_params'], {'permissions': PERMISSIONS})
 
+    def test_test_iam_permissions_w_user_project(self):
+        from google.cloud.storage.iam import STORAGE_OBJECTS_LIST
+        from google.cloud.storage.iam import STORAGE_BUCKETS_GET
+        from google.cloud.storage.iam import STORAGE_BUCKETS_UPDATE
+
+        NAME = 'name'
+        USER_PROJECT = 'user-project-123'
+        PATH = '/b/%s' % (NAME,)
+        PERMISSIONS = [
+            STORAGE_OBJECTS_LIST,
+            STORAGE_BUCKETS_GET,
+            STORAGE_BUCKETS_UPDATE,
+        ]
+        ALLOWED = PERMISSIONS[1:]
+        RETURNED = {'permissions': ALLOWED}
+        connection = _Connection(RETURNED)
+        client = _Client(connection, None)
+        bucket = self._make_one(
+            client=client, name=NAME, user_project=USER_PROJECT)
+
+        allowed = bucket.test_iam_permissions(PERMISSIONS)
+
+        self.assertEqual(allowed, ALLOWED)
+
+        kw = connection._requested
+        self.assertEqual(len(kw), 1)
+        self.assertEqual(kw[0]['method'], 'GET')
+        self.assertEqual(kw[0]['path'], '%s/iam/testPermissions' % (PATH,))
+        self.assertEqual(
+            kw[0]['query_params'],
+            {'permissions': PERMISSIONS, 'userProject': USER_PROJECT})
+
     def test_make_public_defaults(self):
         from google.cloud.storage.acl import _ACLEntity
 
